// Dynamic link library implementation of BackTanhAxon with user gain control

#include "NSDLL.h"

/***********************************/
/* Forward activation of component */

__declspec(dllexport) void performLinearAxon(
	DLLData	*instance,	// Pointer to instance data
	NSFloat	*data, 		// Pointer to the layer of processing elements (PEs)
	int 	rows,		// Number of rows of PEs in the layer
	int 	cols,		// Number of columns of PEs in the layer
	NSFloat	*bias,		// Pointer to the layer's bias vector, one for each PE
	NSFloat	beta		// Slope gain scalar, same for all PEs
	)
{
	int i, length=rows*cols;
	NSFloat gain = getFloatParameter(instance, 2, 1);

	for (i=0; i<length; i++)
		data[i] = gain*(NSFloat)tanh(beta*data[i] + bias[i]);
}

/******************************************/
/* Management of instance data (OPTIONAL) */

__declspec(dllexport) DLLData *allocLinearAxon(
	DLLData	*oldInstance,	// Pointer to the last instance if reallocating
	int 	rows,		// Number of rows of PEs in the layer
	int 	cols		// Number of columns of PEs in the layer
	)
{
	DLLData *instance = allocDLLInstance(oldInstance);
	setParameterName(instance, 2, 1, "Gain", FALSE);
	setFloatParameter(instance, 2, 1, 1.0f, FALSE);
	return instance;
}

__declspec(dllexport) void freeLinearAxon(DLLData *instance)
{
	freeDLLInstance(instance);
}
