// Dynamic link library implementation of NeuroSolutions Noise component for Uniform setting

#include "NSDLL.h" 

/***************************/
/* Activation of component */
__declspec(dllexport) NSFloat performNoise(
	DLLData	*instance,	// Pointer to instance data (may be NULL)
	NSFloat	variance, 	// Variance set within components inspector
	NSFloat	mean	 	// Mean set within components inspector
	)
{
	NSFloat decay = getFloatParameter(instance, 2, 1);
	NSFloat *varianceDecay = getUserData(instance);
	NSFloat noise = ((NSFloat)sqrt(3*variance * *varianceDecay)*(NSFloat)(((NSFloat)rand()/RAND_MAX)-0.5)+mean);
	*varianceDecay *= decay;
	return noise;
}

/**********************************************************************/
/* Called before any performNoise calls, allowing any initialization. */
__declspec(dllexport) void getReadyToFire(
	DLLData	*instance	// Pointer to instance data (may be NULL) 
	)
{
	NSFloat *varianceDecay = getUserData(instance);
	*varianceDecay = 1.0f;
}

/******************************************/
/* Management of instance data (OPTIONAL) */
__declspec(dllexport) DLLData *allocNoise(
	DLLData	*oldInstance	// Pointer to the last instance if reallocating 
	)
{
	DLLData *instance = allocDLLInstance(oldInstance);
	setParameterName(instance, 2, 1, "Decay", FALSE);
	setFloatParameter(instance, 2, 1, 0.9f, FALSE);
	setUserData(instance, malloc(sizeof(NSFloat)));
	return instance;
}

__declspec(dllexport) void freeNoise(DLLData *instance)
{
	if (getUserData(instance))
		free(getUserData(instance));
	freeDLLInstance(instance);
}
