// Dynamic link library implementation of a running average processor

#include "NSDLL.h" 

#define	buffer(i, j)	bufferData->data[j+(i)*bufferData->length]

typedef struct {
	int length;
	NSFloat *data;
} AverageData;

/***************************/
/* Activation of component */
__declspec(dllexport) BOOL performPrePost(
	DLLData	*instance,	// Pointer to instance data (may be NULL)
	NSFloat	*input, 	// Pointer to the input data
	NSFloat	*output, 	// Pointer to the output data
	int rows, 		// Number of rows of data
	int cols, 		// Number of cols of data
	BOOL preprocessor	// Flag to indicate whether this is a preprocessor or postprocessor
	)
{
	int i, j;
	int N = getIntParameter(instance, 2, 1);
	AverageData *bufferData = getUserData(instance);
	NSFloat result;

	for (i=N-1; i>0; i--)
		for (j=0; j<bufferData->length; j++) 
			buffer(i,j) = buffer(i-1,j);
	for (j=0; j<bufferData->length; j++)
		buffer(0,j) = input[j];
	if (!preprocessor)
		for (j=0; j<bufferData->length; j++)
			output[j] = 0.0f;
	for (j=0; j<bufferData->length; j++) {
		result = 0.0f;
		for (i=0; i<N; i++)
			result += buffer(i,j);
		output[j] += result/N;
	}
	return TRUE;
}

/******************************************/
/* Called every time the network is reset */
__declspec(dllexport) void networkReset(
	DLLData	*instance	// Pointer to instance data (may be NULL) 
	)
{
	int i, j;
	int N = getIntParameter(instance, 2, 1);
	AverageData *bufferData = getUserData(instance);

	for (i=0; i<N; i++)
		for (j=0; j<bufferData->length; j++) 
			buffer(i,j) = 0.0f;
}

/******************************************/
/* Management of instance data (OPTIONAL) */
__declspec(dllexport) DLLData *allocPrePost(
	DLLData	*oldInstance,	// Pointer to the last instance if reallocating
	int *rows, 		// Number of rows of output data, can be changed to reflect a diffenent number for the input data
	int *cols, 		// Number of cols of output data, can be changed to reflect a diffenent number for the input data
	BOOL preprocessor	// Flag to indicate whether this is a preprocessor or postprocessor
	)
{
	DLLData *instance = allocDLLInstance(oldInstance);
	AverageData *bufferData = calloc(1,sizeof(AverageData));
	setParameterName(instance, 2, 1, "N", TRUE);
	setIntParameter(instance, 2, 1, 4, FALSE);
	bufferData->length = *rows * *cols;
	bufferData->data = calloc(bufferData->length*getIntParameter(instance, 2, 1), sizeof(NSFloat));
	setUserData(instance, bufferData);
	return instance;
}

__declspec(dllexport) void freePrePost(DLLData *instance)
{
	AverageData *bufferData = getUserData(instance);
	if (bufferData) {
		free(bufferData->data);
		free(bufferData);
	}
	freeDLLInstance(instance); 
}
