// Dynamic link library implementation of NeuroSolutions BackSynapse component 

#include "NSDLL.h"

/*************************************************************/
/* Macros to access the PE layers and weights in matrix form */

#define in(i,j)		errorIn[j+i*inCols]
#define out(i,j)	errorOut[j+i*outCols]
#define activity(i,j)	input[j+i*outCols]

/********************************/
/* Backpropagation of component */

__declspec(dllexport) void performBackSynapse(
	DLLData	*instance,		// Pointer to instance data
	DLLData	*dualInstance,	// Pointer to the forward synapses instance data
	NSFloat	*errorIn, 		// Pointer to the input error layer of processing elements (PEs)
	int 	inRows,			// Number of rows of PEs in the input layer (don't forget that input and output are reversed)
	int 	inCols,			// Number of columns of PEs in the input layer
	NSFloat	*errorOut, 		// Pointer to the output error layer 
	int 	outRows,		// Number of rows of PEs in the output layer
	int 	outCols,		// Number of columns of PEs in the output layer
	NSFloat	*input	 		// Pointer to the output layer of the forward synapse
	)
{
	int	i, j, k, l, startRow, stopRow, startCol, stopCol,
		colSize = (int)outCols/inCols,
		colRemainder = outCols-inCols*colSize,
		rowSize = (int)outRows/inRows,
		rowRemainder = outRows-inRows*rowSize;
	NSFloat *weights = getWeights(dualInstance);
	NSFloat *gradients = getWeights(instance);

	for (i=0; i<inRows; i++)
		for (j=0; j<inCols; j++) {
			startRow = rowSize*i;
			if (i == inRows-1)
				stopRow = outRows;
			else
				stopRow = rowSize*(i+1);
			startCol = colSize*j;
			if (j == inCols-1)
				stopCol = outCols;
			else
				stopCol = colSize*(j+1);
			for (k=startRow; k<stopRow; k++)
				for (l=startCol; l<stopCol; l++) {
					out(k,l) += *weights++ * in(i,j);
					*gradients++ += in(i,j) * activity(k,l);
				}
		}
}

/******************************************/
/* Management of instance data (OPTIONAL) */

__declspec(dllexport) DLLData *allocBackSynapse(
	DLLData	*oldInstance,	// Pointer to the last instance if reallocating
	DLLData	*dualInstance,	// Pointer to forward axons instance data (may be NULL)
	int 	inRows,			// Number of rows of PEs in the input layer
	int 	inCols,			// Number of columns of PEs in the input layer
	int 	outRows,		// Number of rows of PEs in the output layer
	int 	outCols			// Number of columns of PEs in the output layer
	)
{
	DLLData *instance = allocDLLInstance(oldInstance);
	setWeights(instance, dualInstance->weights->length);
	return instance;
}

__declspec(dllexport) void freeBackSynapse(DLLData *instance)
{
	freeDLLInstance(instance);
}
