// Dynamic link library implementation of NeuroSolutions Synapse component 

#include "NSDLL.h"

/*************************************************/
/* Macros to access the PE layers in matrix form */

#define in(i,j)		input[j+i*inCols]
#define out(i,j)	output[j+i*outCols]

/***********************************/
/* Forward activation of component */

__declspec(dllexport) void performSynapse(
	DLLData	*instance,	// Pointer to instance data
	NSFloat	*input, 	// Pointer to the input layer of processing elements (PEs)
	int 	inRows,		// Number of rows of PEs in the input layer
	int 	inCols,		// Number of columns of PEs in the input layer
	NSFloat	*output, 	// Pointer to the output layer
	int 	outRows,	// Number of rows of PEs in the output layer
	int 	outCols		// Number of columns of PEs in the output layer
	)
{
	int	i, j, k, l, startRow, stopRow, startCol, stopCol, w=0,
		colSize = (int)inCols/outCols,
		colRemainder = inCols-outCols*colSize,
		rowSize = (int)inRows/outRows,
		rowRemainder = inRows-outRows*rowSize;
	NSFloat *weights = getWeights(instance);

	for (i=0; i<outRows; i++)
		for (j=0; j<outCols; j++) {
			startRow = rowSize*i;
			if (i == outRows-1)
				stopRow = inRows;
			else
				stopRow = rowSize*(i+1);
			startCol = colSize*j;
			if (j == outCols-1)
				stopCol = inCols;
			else
				stopCol = colSize*(j+1);
			for (k=startRow; k<stopRow; k++)
				for (l=startCol; l<stopCol; l++)
					out(i,j) += *weights++ * in(k,l);
		}
}

/******************************************/
/* Management of instance data (OPTIONAL) */

__declspec(dllexport) DLLData *allocSynapse(
	DLLData	*oldInstance,	// Pointer to the last instance if reallocating
	int 	inRows,			// Number of rows of PEs in the input layer
	int 	inCols,			// Number of columns of PEs in the input layer
	int 	outRows,		// Number of rows of PEs in the output layer
	int 	outCols			// Number of columns of PEs in the output layer
	)
{
	DLLData *instance = allocDLLInstance(oldInstance);
	int	colSize = (int)inCols/outCols,
		colRemainder = inCols-outCols*colSize,
		rowSize = (int)inRows/outRows,
		rowRemainder = inRows-outRows*rowSize,
		totalWeights = outRows*outCols*rowSize*colSize + outRows*rowSize*colRemainder + outCols*colSize*rowRemainder + rowRemainder*colRemainder;

	setWeights(instance, totalWeights);
	return instance;
}

__declspec(dllexport) void freeSynapse(DLLData *instance)
{
	freeDLLInstance(instance);
}
